@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: bintrans.c,v 1.1 2005/08/29 14:36:41 debug Exp $
+ *  $Id: bintrans.c,v 1.4 2005/12/26 12:32:10 debug Exp $
  *
  *  Dynamic binary translation.
  *
@@ -714,8 +714,8 @@ cpu->cd.mips.pc_last_host_4k_page,(long long)paddr);
 					/*  tlbr:  */
 					translated = try_to_translate = bintrans_write_instruction__tlb_rfe_etc(&ca, CALL_TLBR);
 					n_translated += translated;
-				} else if (instr[2] == 0 && instr[1] == 0 && (instr[0] == 0x21 || instr[0] == 0x22)) {
-					/*  standby and suspend on VR41xx etc ==> NOP  */
+				} else if (instr[2] == 0 && instr[1] == 0 && (instr[0] == 0x20 || instr[0] == 0x21 || instr[0] == 0x22)) {
+					/*  idle, standby and suspend on VR41xx etc ==> NOP  */
 					translated = try_to_translate = bintrans_write_instruction__addu_etc(cpu->mem, &ca, 0, 0, 0, 0, SPECIAL_SLL);
 					n_translated += translated;
 				} else {
@@ -1101,8 +1101,8 @@ void old_bintrans_init_cpu(struct cpu *cpu)
 #if 1
 		for (i=0; i<1048576; i++) {
 			unsigned char *ptr = NULL;
-			if (i < (0xa0000000ULL >> 12) ||
-			    i >= (0xc0000000ULL >> 12))
+			if ((unsigned int)i < (0xa0000000ULL >> 12) ||
+			    (unsigned int)i >= (0xc0000000ULL >> 12))
 				ptr = cpu->cd.mips.
 				    vaddr_to_hostaddr_r2k3k_dcachetable
 				    ->haddr_entry[(i & 1023) * 2];
@@ -1208,7 +1208,7 @@ void bintrans_init(struct machine *machine, struct memory *mem)
 		}
 	}
 
-	debug("bintrans: "BACKEND_NAME", %i MB translation cache at %p\n",
+	debug("OLD bintrans: "BACKEND_NAME", %i MB translation cache at %p\n",
 	    (int)(s/1048576), mem->translation_code_chunk_space);
 
 	/*
