@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: cpu_arm_instr_dpi.c,v 1.10 2005/10/09 21:32:07 debug Exp $
+ *  $Id: cpu_arm_instr_dpi.c,v 1.16 2005/11/19 18:53:07 debug Exp $
  *
  *
  *  ARM Data Processing Instructions
@@ -69,6 +69,14 @@
  */
 void A__NAME(struct cpu *cpu, struct arm_instr_call *ic)
 {
+#if defined(A__RSB) || defined(A__RSC)
+#define VAR_A  b
+#define VAR_B  a
+#else
+#define VAR_A  a
+#define VAR_B  b
+#endif
+
 #ifdef A__REG
 	uint32_t (*reg_func)(struct cpu *, struct arm_instr_call *)
 	    = (void *)(size_t)ic->arg[1];
@@ -87,17 +95,21 @@ void A__NAME(struct cpu *cpu, struct arm_instr_call *ic)
 #else
 	uint32_t
 #endif
-	    c64,
-#if !defined(A__MOV) && !defined(A__MVN)
-	    a = reg(ic->arg[0]),
-#endif
-	    b =
+	    VAR_B =
 #ifdef A__REG
-	    reg_func(cpu, ic);
+	    reg_func(cpu, ic)
 #else
-	    ic->arg[1];
+#ifdef A__REGSHORT
+	    reg(ic->arg[1])
+#else
+	    ic->arg[1]
 #endif
-
+#endif
+	    , c64
+#if !defined(A__MOV) && !defined(A__MVN)
+	    , VAR_A = reg(ic->arg[0])
+#endif
+	    ;
 
 #if defined(A__MOV) || defined(A__MVN) || defined(A__TST) || defined(A__TEQ) \
  || defined(A__AND) || defined(A__BIC) || defined(A__EOR) || defined(A__ORR)
@@ -111,10 +123,11 @@ void A__NAME(struct cpu *cpu, struct arm_instr_call *ic)
 	 *  TODO 2: Perhaps this check should be moved out from here, and into
 	 *  cpu_arm_instr.c. (More correct, and higher performance.)
 	 */
-	if (b > 255) {
-		cpu->cd.arm.cpsr &= ~ARM_FLAG_C;
-		if (b & 0x80000000)
-			cpu->cd.arm.cpsr |= ARM_FLAG_C;
+	if (VAR_B > 255) {
+		if (VAR_B & 0x80000000)
+			cpu->cd.arm.flags |= ARM_F_C;
+		else
+			cpu->cd.arm.flags &= ~ARM_F_C;
 	}
 #endif
 #endif
@@ -123,21 +136,15 @@ void A__NAME(struct cpu *cpu, struct arm_instr_call *ic)
 #if !defined(A__MOV) && !defined(A__MVN)
 #ifdef A__PC
 	if (ic->arg[0] == (size_t)&cpu->cd.arm.r[ARM_PC]) {
-		uint32_t low_pc;
-		low_pc = ((size_t)ic - (size_t)
+		uint32_t low_pc = ((size_t)ic - (size_t)
 		    cpu->cd.arm.cur_ic_page) / sizeof(struct arm_instr_call);
-		a = cpu->pc & ~((ARM_IC_ENTRIES_PER_PAGE-1)
+		VAR_A = cpu->pc & ~((ARM_IC_ENTRIES_PER_PAGE-1)
 		    << ARM_INSTR_ALIGNMENT_SHIFT);
-		a += (low_pc << ARM_INSTR_ALIGNMENT_SHIFT) + 8;
+		VAR_A += (low_pc << ARM_INSTR_ALIGNMENT_SHIFT) + 8;
 	}
 #endif
 #endif
 
-
-#if defined(A__RSB) || defined(A__RSC)
-	{ uint32_t tmp = a; a = b; b = tmp; }
-#endif
-
 	/*
 	 *  Perform the operation:
 	 */
@@ -154,10 +161,11 @@ void A__NAME(struct cpu *cpu, struct arm_instr_call *ic)
 	c64 = a + b;
 #endif
 #if defined(A__ADC)
-	c64 = a + b + (cpu->cd.arm.cpsr & ARM_FLAG_C? 1 : 0);
+	c64 = a + b + (cpu->cd.arm.flags & ARM_F_C? 1 : 0);
 #endif
 #if defined(A__SBC) || defined(A__RSC)
-	c64 = a - b - (1 - (cpu->cd.arm.cpsr & ARM_FLAG_C? 1 : 0));
+	b += (cpu->cd.arm.flags & ARM_F_C? 0 : 1);
+	c64 = a - b;
 #endif
 #if defined(A__ORR)
 	c64 = a | b;
@@ -179,12 +187,12 @@ void A__NAME(struct cpu *cpu, struct arm_instr_call *ic)
 #ifdef A__PC
 	if (ic->arg[2] == (size_t)&cpu->cd.arm.r[ARM_PC]) {
 #ifndef A__S
-		uint32_t old_pc = cpu->cd.arm.r[ARM_PC];
+		uint32_t old_pc = cpu->pc;
 		uint32_t mask_within_page = ((ARM_IC_ENTRIES_PER_PAGE-1)
 		    << ARM_INSTR_ALIGNMENT_SHIFT) |
 		    ((1 << ARM_INSTR_ALIGNMENT_SHIFT) - 1);
 #endif
-		cpu->pc = reg(ic->arg[2]) = c64;
+		cpu->pc = (uint32_t)c64;
 #ifdef A__S
 		/*  Copy the right SPSR into CPSR:  */
 		arm_save_register_bank(cpu);
@@ -199,19 +207,18 @@ void A__NAME(struct cpu *cpu, struct arm_instr_call *ic)
 			cpu->cd.arm.cpsr = cpu->cd.arm.spsr_abt; break;
 		case ARM_MODE_UND32:
 			cpu->cd.arm.cpsr = cpu->cd.arm.spsr_und; break;
-		default:fatal("huh? weird mode in dpi s with pc\n");
-			exit(1);
 		}
+		cpu->cd.arm.flags = cpu->cd.arm.cpsr >> 28;
 		arm_load_register_bank(cpu);
 #else
 		if ((old_pc & ~mask_within_page) ==
-		    (cpu->pc & ~mask_within_page)) {
+		    ((uint32_t)cpu->pc & ~mask_within_page)) {
 			cpu->cd.arm.next_ic = cpu->cd.arm.cur_ic_page +
 			    ((cpu->pc & mask_within_page) >>
 			    ARM_INSTR_ALIGNMENT_SHIFT);
 		} else
 #endif
-			arm_pc_to_pointers(cpu);
+			quick_pc_to_pointers(cpu);
 		return;
 	} else
 		reg(ic->arg[2]) = c64;
@@ -226,41 +233,30 @@ void A__NAME(struct cpu *cpu, struct arm_instr_call *ic)
 	 */
 #ifdef A__S
 	c32 = c64;
-	cpu->cd.arm.cpsr &= ~(ARM_FLAG_Z | ARM_FLAG_N
+	cpu->cd.arm.flags
 #if defined(A__CMP) || defined(A__CMN) || defined(A__ADC) || defined(A__ADD) \
  || defined(A__RSB) || defined(A__RSC) || defined(A__SBC) || defined(A__SUB)
-	    | ARM_FLAG_V | ARM_FLAG_C
+	    = 0;
+#else
+	    &= ~(ARM_F_Z | ARM_F_N);
 #endif
-	    );
 
-#if defined(A__CMP) || defined(A__RSB) || defined(A__SUB)
+#if defined(A__CMP) || defined(A__RSB) || defined(A__SUB) || \
+    defined(A__RSC) || defined(A__SBC)
 	if ((uint32_t)a >= (uint32_t)b)
-		cpu->cd.arm.cpsr |= ARM_FLAG_C;
+		cpu->cd.arm.flags |= ARM_F_C;
 #else
 #if defined(A__ADC) || defined(A__ADD) || defined(A__CMN)
 	if (c32 != c64)
-		cpu->cd.arm.cpsr |= ARM_FLAG_C;
-#else
-#if defined(A__RSC) || defined(A__SBC)
-	{
-		uint32_t low_pc;
-		low_pc = ((size_t)ic - (size_t)
-		    cpu->cd.arm.cur_ic_page) / sizeof(struct arm_instr_call);
-		a = cpu->pc & ~((ARM_IC_ENTRIES_PER_PAGE-1)
-		    << ARM_INSTR_ALIGNMENT_SHIFT);
-		a += (low_pc << ARM_INSTR_ALIGNMENT_SHIFT) + 8;
-		fatal("TODO: C flag: pc = 0x%08x\n", a);
-		exit(1);
-	}
-#endif
+		cpu->cd.arm.flags |= ARM_F_C;
 #endif
 #endif
 
 	if (c32 == 0)
-		cpu->cd.arm.cpsr |= ARM_FLAG_Z;
+		cpu->cd.arm.flags |= ARM_F_Z;
 
 	if ((int32_t)c32 < 0)
-		cpu->cd.arm.cpsr |= ARM_FLAG_N;
+		cpu->cd.arm.flags |= ARM_F_N;
 
 	/*  Calculate the Overflow bit:  */
 #if defined(A__CMP) || defined(A__CMN) || defined(A__ADC) || defined(A__ADD) \
@@ -274,60 +270,60 @@ void A__NAME(struct cpu *cpu, struct arm_instr_call *ic)
 		    (int32_t)c32 >= 0))
 			v = 1;
 #else
-#if defined(A__SUB) || defined(A__RSB) || defined(A__CMP)
+#if defined(A__SUB) || defined(A__RSB) || defined(A__CMP) || \
+    defined(A__RSC) || defined(A__SBC)
 		if (((int32_t)a >= 0 && (int32_t)b < 0 &&
 		    (int32_t)c32 < 0) ||
 		    ((int32_t)a < 0 && (int32_t)b >= 0 &&
 		    (int32_t)c32 >= 0))
 			v = 1;
-#else
-		fatal("NO\n");
-		exit(1);
 #endif
 #endif
 		if (v)
-			cpu->cd.arm.cpsr |= ARM_FLAG_V;
+			cpu->cd.arm.flags |= ARM_F_V;
 	}
 #endif
 #endif	/*  A__S  */
+
+#undef VAR_A
+#undef VAR_B
 }
 
 
 void A__NAME__eq(struct cpu *cpu, struct arm_instr_call *ic)
-{ if (cpu->cd.arm.cpsr & ARM_FLAG_Z) A__NAME(cpu, ic); }
+{ if (cpu->cd.arm.flags & ARM_F_Z) A__NAME(cpu, ic); }
 void A__NAME__ne(struct cpu *cpu, struct arm_instr_call *ic)
-{ if (!(cpu->cd.arm.cpsr & ARM_FLAG_Z)) A__NAME(cpu, ic); }
+{ if (!(cpu->cd.arm.flags & ARM_F_Z)) A__NAME(cpu, ic); }
 void A__NAME__cs(struct cpu *cpu, struct arm_instr_call *ic)
-{ if (cpu->cd.arm.cpsr & ARM_FLAG_C) A__NAME(cpu, ic); }
+{ if (cpu->cd.arm.flags & ARM_F_C) A__NAME(cpu, ic); }
 void A__NAME__cc(struct cpu *cpu, struct arm_instr_call *ic)
-{ if (!(cpu->cd.arm.cpsr & ARM_FLAG_C)) A__NAME(cpu, ic); }
+{ if (!(cpu->cd.arm.flags & ARM_F_C)) A__NAME(cpu, ic); }
 void A__NAME__mi(struct cpu *cpu, struct arm_instr_call *ic)
-{ if (cpu->cd.arm.cpsr & ARM_FLAG_N) A__NAME(cpu, ic); }
+{ if (cpu->cd.arm.flags & ARM_F_N) A__NAME(cpu, ic); }
 void A__NAME__pl(struct cpu *cpu, struct arm_instr_call *ic)
-{ if (!(cpu->cd.arm.cpsr & ARM_FLAG_N)) A__NAME(cpu, ic); }
+{ if (!(cpu->cd.arm.flags & ARM_F_N)) A__NAME(cpu, ic); }
 void A__NAME__vs(struct cpu *cpu, struct arm_instr_call *ic)
-{ if (cpu->cd.arm.cpsr & ARM_FLAG_V) A__NAME(cpu, ic); }
+{ if (cpu->cd.arm.flags & ARM_F_V) A__NAME(cpu, ic); }
 void A__NAME__vc(struct cpu *cpu, struct arm_instr_call *ic)
-{ if (!(cpu->cd.arm.cpsr & ARM_FLAG_V)) A__NAME(cpu, ic); }
+{ if (!(cpu->cd.arm.flags & ARM_F_V)) A__NAME(cpu, ic); }
+
+#ifndef BLAHURG
+#define BLAHURG
+extern uint8_t condition_hi[16];
+extern uint8_t condition_ge[16];
+extern uint8_t condition_gt[16];
+#endif
 
 void A__NAME__hi(struct cpu *cpu, struct arm_instr_call *ic)
-{ if (cpu->cd.arm.cpsr & ARM_FLAG_C &&
-!(cpu->cd.arm.cpsr & ARM_FLAG_Z)) A__NAME(cpu, ic); }
+{ if (condition_hi[cpu->cd.arm.flags]) A__NAME(cpu, ic); }
 void A__NAME__ls(struct cpu *cpu, struct arm_instr_call *ic)
-{ if (cpu->cd.arm.cpsr & ARM_FLAG_Z ||
-!(cpu->cd.arm.cpsr & ARM_FLAG_C)) A__NAME(cpu, ic); }
+{ if (!condition_hi[cpu->cd.arm.flags]) A__NAME(cpu, ic); }
 void A__NAME__ge(struct cpu *cpu, struct arm_instr_call *ic)
-{ if (((cpu->cd.arm.cpsr & ARM_FLAG_N)?1:0) ==
-((cpu->cd.arm.cpsr & ARM_FLAG_V)?1:0)) A__NAME(cpu, ic); }
+{ if (condition_ge[cpu->cd.arm.flags]) A__NAME(cpu, ic); }
 void A__NAME__lt(struct cpu *cpu, struct arm_instr_call *ic)
-{ if (((cpu->cd.arm.cpsr & ARM_FLAG_N)?1:0) !=
-((cpu->cd.arm.cpsr & ARM_FLAG_V)?1:0)) A__NAME(cpu, ic); }
+{ if (!condition_ge[cpu->cd.arm.flags]) A__NAME(cpu, ic); }
 void A__NAME__gt(struct cpu *cpu, struct arm_instr_call *ic)
-{ if (((cpu->cd.arm.cpsr & ARM_FLAG_N)?1:0) ==
-((cpu->cd.arm.cpsr & ARM_FLAG_V)?1:0) &&
-!(cpu->cd.arm.cpsr & ARM_FLAG_Z)) A__NAME(cpu, ic); }
+{ if (condition_gt[cpu->cd.arm.flags]) A__NAME(cpu, ic); }
 void A__NAME__le(struct cpu *cpu, struct arm_instr_call *ic)
-{ if (((cpu->cd.arm.cpsr & ARM_FLAG_N)?1:0) !=
-((cpu->cd.arm.cpsr & ARM_FLAG_V)?1:0) ||
-(cpu->cd.arm.cpsr & ARM_FLAG_Z)) A__NAME(cpu, ic); }
+{ if (!condition_gt[cpu->cd.arm.flags]) A__NAME(cpu, ic); }
 
