@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: cpu_avr.c,v 1.4 2005/09/17 22:34:52 debug Exp $
+ *  $Id: cpu_avr.c,v 1.7 2005/11/13 00:14:07 debug Exp $
  *
  *  Atmel AVR (8-bit) CPU emulation.
  */
@@ -43,6 +43,7 @@
 
 
 #define	DYNTRANS_32
+#define	DYNTRANS_VARIABLE_INSTRUCTION_LENGTH
 #include "tmp_avr_head.c"
 
 
@@ -62,8 +63,8 @@ int avr_cpu_new(struct cpu *cpu, struct memory *mem, struct machine *machine,
 
 	cpu->memory_rw = avr_memory_rw;
 	cpu->update_translation_table = avr_update_translation_table;
-	cpu->invalidate_translation_caches_paddr =
-	    avr_invalidate_translation_caches_paddr;
+	cpu->invalidate_translation_caches =
+	    avr_invalidate_translation_caches;
 	cpu->invalidate_code_translation = avr_invalidate_code_translation;
 	cpu->is_32bit = 1;
 
@@ -178,32 +179,6 @@ void avr_cpu_register_match(struct machine *m, char *name,
 }
 
 
-/*
- *  avr_cpu_show_full_statistics():
- *
- *  Show detailed statistics on opcode usage on each cpu.
- */
-void avr_cpu_show_full_statistics(struct machine *m)
-{
-	fatal("avr_cpu_show_full_statistics(): TODO\n");
-}
-
-
-/*
- *  avr_cpu_tlbdump():
- *
- *  Called from the debugger to dump the TLB in a readable format.
- *  x is the cpu number to dump, or -1 to dump all CPUs.
- *
- *  If rawflag is nonzero, then the TLB contents isn't formated nicely,
- *  just dumped.
- */
-void avr_cpu_tlbdump(struct machine *m, int x, int rawflag)
-{
-	fatal("avr_cpu_tlbdump(): TODO\n");
-}
-
-
 /*
  *  avr_cpu_interrupt():
  */
