@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2005  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2005-2006  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: cpu_avr_instr.c,v 1.4 2005/11/06 22:41:12 debug Exp $
+ *  $Id: cpu_avr_instr.c,v 1.6 2006/02/09 22:40:27 debug Exp $
  *
  *  Atmel AVR (8-bit) instructions.
  *
@@ -199,6 +199,9 @@ void avr_combine_instructions(struct cpu *cpu, struct avr_instr_call *ic,
 X(to_be_translated)
 {
 	int addr, low_pc, rd, rr, main_opcode;
+#ifdef DYNTRANS_BACKEND
+	int simple = 0;
+#endif
 	uint16_t iword;
 	unsigned char *page;
 	unsigned char ib[2];
