@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2005  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2005-2006  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: cpu_sh_instr.c,v 1.3 2005/11/06 22:41:12 debug Exp $
+ *  $Id: cpu_sh_instr.c,v 1.5 2006/02/09 22:40:27 debug Exp $
  *
  *  SH instructions.
  *
@@ -80,6 +80,9 @@ X(to_be_translated)
 	uint32_t iword;
 	unsigned char *page;
 	unsigned char ib[4];
+#ifdef DYNTRANS_BACKEND
+	int simple = 0;
+#endif
 	int main_opcode, instr_size = 4;
 	/* void (*samepage_function)(struct cpu *, struct sh_instr_call *);*/
 
