@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2005  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2005-2006  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: cpu_sparc_instr.c,v 1.3 2005/11/06 22:41:12 debug Exp $
+ *  $Id: cpu_sparc_instr.c,v 1.5 2006/02/09 22:40:27 debug Exp $
  *
  *  SPARC instructions.
  *
@@ -77,6 +77,9 @@ X(end_of_page)
 X(to_be_translated)
 {
 	uint64_t addr, low_pc;
+#ifdef DYNTRANS_BACKEND
+	int simple = 0;
+#endif
 	uint32_t iword;
 	unsigned char *page;
 	unsigned char ib[4];
