@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2005  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2005-2006  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: cpu_x86_instr.c,v 1.2 2005/11/06 22:41:12 debug Exp $
+ *  $Id: cpu_x86_instr.c,v 1.5 2006/02/09 22:55:20 debug Exp $
  *
  *  x86/amd64 instructions.
  *
@@ -76,10 +76,13 @@ X(end_of_page)
 X(to_be_translated)
 {
 	uint64_t addr, low_pc;
+#ifdef DYNTRANS_BACKEND
+	int simple = 0;
+#endif
 	unsigned char *page;
 	int main_opcode;
 	unsigned char ib[17];
-	void (*samepage_function)(struct cpu *, struct x86_instr_call *);
+	/* void (*samepage_function)(struct cpu *, struct x86_instr_call *); */
 
 	/*  Figure out the (virtual) address of the instruction:  */
 	low_pc = ((size_t)ic - (size_t)cpu->cd.x86.cur_ic_page)
