@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: autodev_head.c,v 1.5 2005/11/08 11:01:46 debug Exp $
+ *  $Id: autodev_head.c,v 1.6 2005/12/20 21:19:15 debug Exp $
  *
  *  Automatically register all devices from the devices/ subdir.
  *
@@ -34,7 +34,7 @@
  *  combined into one. See makeautodev.sh for more info.
  */
 
-#include "stdio.h"
+#include <stdio.h>
 
 #include "bus_pci.h"
 #include "device.h"
