@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2005  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2004-2006  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_8250.c,v 1.18 2005/11/13 00:14:08 debug Exp $
+ *  $Id: dev_8250.c,v 1.22 2006/02/18 13:42:39 debug Exp $
  *  
  *  8250 serial controller.
  *
@@ -45,14 +45,16 @@
 
 
 struct dev_8250_data {
-	int		reg[8];
-
 	int		console_handle;
+	char		*name;
+
 	int		irq_enable;
 	int		irqnr;
-
+	int		in_use;
 	int		addrmult;
 
+	int		reg[8];
+
 	int		dlab;		/*  Divisor Latch Access bit  */
 	int		divisor;
 	int		databits;
@@ -61,6 +63,7 @@ struct dev_8250_data {
 };
 
 #define	DEV_8250_LENGTH		8
+#define	DEV_8250_TICKSHIFT	15
 
 
 /*
@@ -100,8 +103,7 @@ void dev_8250_tick(struct cpu *cpu, void *extra)
 /*
  *  dev_8250_access():
  */
-int dev_8250_access(struct cpu *cpu, struct memory *mem, uint64_t relative_addr,
-	unsigned char *data, size_t len, int writeflag, void *extra)
+DEVICE_ACCESS(8250)
 {
 	uint64_t idata = 0, odata = 0;
 	struct dev_8250_data *d = extra;
@@ -111,12 +113,11 @@ int dev_8250_access(struct cpu *cpu, struct memory *mem, uint64_t relative_addr,
 
 	relative_addr /= d->addrmult;
 
-	if (writeflag == MEM_WRITE && relative_addr == 0)
+	if (writeflag == MEM_WRITE && relative_addr == 0) {
 		console_putchar(d->console_handle, idata);
-	else
-	if (writeflag == MEM_READ && relative_addr == 5)
+	} else if (writeflag == MEM_READ && relative_addr == 5) {
 		odata = 64 + 32;
-	else {
+	} else {
 #if 0
 		if (writeflag == MEM_WRITE)
 			fatal("[ 8250: write addr=0x%02x idata = 0x%02x ]\n",
@@ -133,11 +134,10 @@ int dev_8250_access(struct cpu *cpu, struct memory *mem, uint64_t relative_addr,
 }
 
 
-/*
- *  devinit_8250():
- */
-int devinit_8250(struct devinit *devinit)
+DEVINIT(8250)
 {
+	size_t nlen;
+	char *name;
 	struct dev_8250_data *d;
 
 	d = malloc(sizeof(struct dev_8250_data));
@@ -146,19 +146,37 @@ int devinit_8250(struct devinit *devinit)
 		exit(1);
 	}
 	memset(d, 0, sizeof(struct dev_8250_data));
-	d->irqnr = devinit->irq_nr;
+	d->irqnr    = devinit->irq_nr;
 	d->addrmult = devinit->addr_mult;
-	d->console_handle = console_start_slave(devinit->machine, "console");
-	d->dlab = 0;
+	d->in_use   = devinit->in_use;
+	d->dlab     = 0;
 	d->divisor  = 115200 / 9600;
 	d->databits = 8;
 	d->parity   = 'N';
 	d->stopbits = "1";
+	d->name = devinit->name2 != NULL? devinit->name2 : "";
+	d->console_handle =
+	    console_start_slave(devinit->machine, devinit->name2 != NULL?
+	    devinit->name2 : devinit->name, d->in_use);
 
-	memory_device_register(devinit->machine->memory, devinit->name,
+	nlen = strlen(devinit->name) + 10;
+	if (devinit->name2 != NULL)
+		nlen += strlen(devinit->name2);
+	name = malloc(nlen);
+	if (name == NULL) {
+		fprintf(stderr, "out of memory\n");
+		exit(1);
+	}
+	if (devinit->name2 != NULL && devinit->name2[0])
+		snprintf(name, nlen, "%s [%s]", devinit->name, devinit->name2);
+	else
+		snprintf(name, nlen, "%s", devinit->name);
+
+	memory_device_register(devinit->machine->memory, name,
 	    devinit->addr, DEV_8250_LENGTH * devinit->addr_mult,
 	    dev_8250_access, d, DM_DEFAULT, NULL);
-	machine_add_tickfunction(devinit->machine, dev_8250_tick, d, 14);
+	machine_add_tickfunction(devinit->machine, dev_8250_tick, d,
+	    DEV_8250_TICKSHIFT);
 
 	return 1;
 }
