@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2005  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2005-2006  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_8253.c,v 1.7 2005/11/13 00:14:08 debug Exp $
+ *  $Id: dev_8253.c,v 1.9 2006/02/09 20:02:58 debug Exp $
  *  
  *  8253/8254 Programmable Interval Timer.
  *
@@ -73,9 +73,7 @@ void dev_8253_tick(struct cpu *cpu, void *extra)
 /*
  *  dev_8253_access():
  */
-int dev_8253_access(struct cpu *cpu, struct memory *mem,
-	uint64_t relative_addr, unsigned char *data, size_t len,
-	int writeflag, void *extra)
+DEVICE_ACCESS(8253)
 {
 	struct pit8253_data *d = (struct pit8253_data *) extra;
 	uint64_t idata = 0, odata = 0;
@@ -123,10 +121,7 @@ int dev_8253_access(struct cpu *cpu, struct memory *mem,
 }
 
 
-/*
- *  devinit_8253():
- */
-int devinit_8253(struct devinit *devinit)
+DEVINIT(8253)
 {
 	struct pit8253_data *d = malloc(sizeof(struct pit8253_data));
 
