@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2005  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2005-2006  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_8259.c,v 1.22 2005/11/23 18:16:42 debug Exp $
+ *  $Id: dev_8259.c,v 1.24 2006/02/09 20:02:58 debug Exp $
  *  
  *  8259 Programmable Interrupt Controller.
  *
@@ -54,9 +54,7 @@
 /*
  *  dev_8259_access():
  */
-int dev_8259_access(struct cpu *cpu, struct memory *mem,
-	uint64_t relative_addr, unsigned char *data, size_t len,
-	int writeflag, void *extra)
+DEVICE_ACCESS(8259)
 {
 	struct pic8259_data *d = (struct pic8259_data *) extra;
 	uint64_t idata = 0, odata = 0;
@@ -251,7 +249,7 @@ int dev_8259_access(struct cpu *cpu, struct memory *mem,
  *	    which the PIC is connected. (That is left to machine specific
  *	    code in src/machine.c.)
  */
-int devinit_8259(struct devinit *devinit)
+DEVINIT(8259)
 {
 	struct pic8259_data *d = malloc(sizeof(struct pic8259_data));
 	char *name2;
