@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2003-2005  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2003-2006  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_dc7085.c,v 1.51 2005/11/13 00:14:08 debug Exp $
+ *  $Id: dev_dc7085.c,v 1.54 2006/01/01 13:17:16 debug Exp $
  *  
  *  DC7085 serial controller, used in some DECstation models.
  */
@@ -164,12 +164,10 @@ void dev_dc7085_tick(struct cpu *cpu, void *extra)
 /*
  *  dev_dc7085_access():
  */
-int dev_dc7085_access(struct cpu *cpu, struct memory *mem,
-	uint64_t relative_addr, unsigned char *data, size_t len,
-	int writeflag, void *extra)
+DEVICE_ACCESS(dc7085)
 {
 	uint64_t idata = 0, odata = 0;
-	int i;
+	size_t i;
 	struct dc_data *d = extra;
 
 	if (writeflag == MEM_WRITE)
@@ -316,7 +314,7 @@ int dev_dc7085_init(struct machine *machine, struct memory *mem,
 	d->regs.dc_csr = CSR_TRDY | CSR_MSE;
 	d->regs.dc_tcr = 0x00;
 
-	d->console_handle = console_start_slave(machine, "DC7085");
+	d->console_handle = console_start_slave(machine, "DC7085", 1);
 
 	lk201_init(&d->lk201, use_fb, add_to_rx_queue, d->console_handle, d);
 
