@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2005  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2005-2006  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_disk.c,v 1.9 2005/11/13 00:14:08 debug Exp $
+ *  $Id: dev_disk.c,v 1.11 2006/02/09 20:02:59 debug Exp $
  *
  *  Basic "Disk" device. This is a simple test device which can be used to
  *  read and write data from disk devices.
@@ -59,9 +59,7 @@ struct disk_data {
 /*
  *  dev_disk_buf_access():
  */
-int dev_disk_buf_access(struct cpu *cpu, struct memory *mem,
-	uint64_t relative_addr, unsigned char *data, size_t len,
-	int writeflag, void *extra)
+DEVICE_ACCESS(disk_buf)
 {
 	struct disk_data *d = (struct disk_data *) extra;
 
@@ -76,9 +74,7 @@ int dev_disk_buf_access(struct cpu *cpu, struct memory *mem,
 /*
  *  dev_disk_access():
  */
-int dev_disk_access(struct cpu *cpu, struct memory *mem,
-	uint64_t relative_addr, unsigned char *data, size_t len,
-	int writeflag, void *extra)
+DEVICE_ACCESS(disk)
 {
 	struct disk_data *d = (struct disk_data *) extra;
 	uint64_t idata = 0, odata = 0;
@@ -142,10 +138,7 @@ int dev_disk_access(struct cpu *cpu, struct memory *mem,
 }
 
 
-/*
- *  devinit_disk():
- */
-int devinit_disk(struct devinit *devinit)
+DEVINIT(disk)
 {
 	struct disk_data *d = malloc(sizeof(struct disk_data));
 	size_t nlen;
