@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2003-2005  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2003-2006  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_le.c,v 1.46 2005/11/13 00:14:09 debug Exp $
+ *  $Id: dev_le.c,v 1.49 2006/01/01 13:17:16 debug Exp $
  *  
  *  LANCE ethernet, as used in DECstations.
  *
@@ -47,6 +47,9 @@
  *
  *  TODO:  Error conditions (such as when there are not enough receive
  *	   buffers) are not emulated yet.
+ *
+ *	   (Old bug, but probably still valid:  "UDP packets that are too 
+ *	   large are not handled well by the Lance device.")
  */
 
 #include <stdio.h>
@@ -230,7 +233,8 @@ static void le_tx(struct net *net, struct le_data *d)
 {
 	int start_txp = d->txp;
 	uint16_t tx_descr[4];
-	int stp, enp, i, cur_packet_offset;
+	int stp, enp, cur_packet_offset;
+	size_t i;
 	uint32_t bufaddr, buflen;
 
 	/*  TODO: This is just a guess:  */
@@ -349,7 +353,8 @@ static void le_tx(struct net *net, struct le_data *d)
  */
 static void le_rx(struct net *net, struct le_data *d)
 {
-	int i, start_rxp = d->rxp;
+	int start_rxp = d->rxp;
+	size_t i;
 	uint16_t rx_descr[4];
 	uint32_t bufaddr, buflen;
 
@@ -387,7 +392,7 @@ static void le_rx(struct net *net, struct le_data *d)
 
 		/*  Copy data from the packet into SRAM:  */
 		for (i=0; i<buflen; i++) {
-			if (d->rx_packet_offset + i >= d->rx_packet_len)
+			if (d->rx_packet_offset+(ssize_t)i >= d->rx_packet_len)
 				break;
 			d->sram[(bufaddr + i) & (SRAM_SIZE-1)] =
 			    d->rx_packet[d->rx_packet_offset + i];
@@ -602,11 +607,10 @@ void le_register_write(struct le_data *d, int r, uint32_t x)
 /*
  *  dev_le_sram_access():
  */
-int dev_le_sram_access(struct cpu *cpu, struct memory *mem,
-	uint64_t relative_addr, unsigned char *data, size_t len,
-	int writeflag, void *extra)
+DEVICE_ACCESS(le_sram)
 {
-	int i, retval;
+	size_t i;
+	int retval;
 	struct le_data *d = extra;
 
 #ifdef LE_DEBUG
@@ -651,11 +655,11 @@ int dev_le_sram_access(struct cpu *cpu, struct memory *mem,
 /*
  *  dev_le_access():
  */
-int dev_le_access(struct cpu *cpu, struct memory *mem, uint64_t relative_addr,
-	unsigned char *data, size_t len, int writeflag, void *extra)
+DEVICE_ACCESS(le)
 {
 	uint64_t idata = 0, odata = 0;
-	int i, retval = 1;
+	size_t i;
+	int retval = 1;
 	struct le_data *d = extra;
 
 	if (writeflag == MEM_WRITE)
@@ -672,11 +676,12 @@ int dev_le_access(struct cpu *cpu, struct memory *mem, uint64_t relative_addr,
 
 	/*  Read from station's ROM (ethernet address):  */
 	if (relative_addr >= 0xc0000 && relative_addr <= 0xfffff) {
-		i = (relative_addr & 0xff) / 4;
-		i = d->rom[i & (ROM_SIZE-1)];
+		uint32_t a;
+		int j = (relative_addr & 0xff) / 4;
+		a = d->rom[j & (ROM_SIZE-1)];
 
 		if (writeflag == MEM_READ) {
-			odata = (i << 24) + (i << 16) + (i << 8) + i;
+			odata = (a << 24) + (a << 16) + (a << 8) + a;
 		} else {
 			fatal("[ le: WRITE to ethernet addr (%08lx):",
 			    (long)relative_addr);
