@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2005  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2004-2006  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_m700_fb.c,v 1.14 2005/11/13 00:14:09 debug Exp $
+ *  $Id: dev_m700_fb.c,v 1.15 2006/01/01 13:17:16 debug Exp $
  *  
  *  Olivetti M700 framebuffer.
  *
@@ -68,9 +68,7 @@ static void schedule_redraw_of_whole_screen(struct m700_fb_data *d)
 /*
  *  dev_m700_fb_access():
  */
-int dev_m700_fb_access(struct cpu *cpu, struct memory *mem,
-	uint64_t relative_addr, unsigned char *data, size_t len,
-	int writeflag, void *extra)
+DEVICE_ACCESS(m700_fb)
 {
 	struct m700_fb_data *d = (struct m700_fb_data *) extra;
 	uint64_t idata = 0, odata = 0;
