@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2005  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2005-2006  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_malta_lcd.c,v 1.4 2005/11/13 00:14:09 debug Exp $
+ *  $Id: dev_malta_lcd.c,v 1.6 2006/02/09 20:02:59 debug Exp $
  *
  *  Malta (evbmips) LCD thingy. Mostly a dummy device.
  */
@@ -80,9 +80,7 @@ void dev_malta_lcd_tick(struct cpu *cpu, void *extra)
 /*
  *  dev_malta_lcd_access():
  */
-int dev_malta_lcd_access(struct cpu *cpu, struct memory *mem,
-	uint64_t relative_addr, unsigned char *data, size_t len,
-	int writeflag, void *extra)
+DEVICE_ACCESS(malta_lcd)
 {
 	struct malta_lcd_data *d = (struct malta_lcd_data *) extra;
 	uint64_t idata = 0, odata = 0;
@@ -124,10 +122,7 @@ int dev_malta_lcd_access(struct cpu *cpu, struct memory *mem,
 }
 
 
-/*
- *  devinit_malta_lcd():
- */
-int devinit_malta_lcd(struct devinit *devinit)
+DEVINIT(malta_lcd)
 {
 	struct malta_lcd_data *d = malloc(sizeof(struct malta_lcd_data));
 
