@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2003-2005  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2003-2006  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_mc146818.c,v 1.80 2005/11/20 11:28:45 debug Exp $
+ *  $Id: dev_mc146818.c,v 1.84 2006/01/17 05:55:53 debug Exp $
  *  
  *  MC146818 real-time clock, used by many different machines types.
  *  (DS1687 as used in some other machines is also similar to the MC146818.)
@@ -319,7 +319,8 @@ int dev_mc146818_access(struct cpu *cpu, struct memory *mem,
 	struct tm *tmp;
 	time_t timet;
 	struct mc_data *d = extra;
-	int i, relative_addr = r;
+	int relative_addr = r;
+	size_t i;
 
 	relative_addr /= d->addrdiv;
 
@@ -506,10 +507,12 @@ int dev_mc146818_access(struct cpu *cpu, struct memory *mem,
 		case 0x128:
 			d->reg[relative_addr] = data[0];
 			if (data[0] & 8) {
+				int j;
+
 				/*  Used on SGI to power off the machine.  */
 				fatal("[ md146818: power off ]\n");
-				for (i=0; i<cpu->machine->ncpus; i++)
-					cpu->machine->cpus[i]->running = 0;
+				for (j=0; j<cpu->machine->ncpus; j++)
+					cpu->machine->cpus[j]->running = 0;
 				cpu->machine->
 				    exit_without_entering_debugger = 1;
 			}
@@ -639,7 +642,7 @@ void dev_mc146818_init(struct machine *machine, struct memory *mem,
 
 	if (machine->machine_type != MACHINE_PREP) {
 		/*  NetBSD/prep has a really ugly clock detection code;
-		    all other machines/OSes don't need this.  */
+		    no other machines/OSes don't need this.  */
 		d->ugly_netbsd_prep_hack_done = NETBSD_HACK_DONE;
 	}
 
@@ -690,7 +693,7 @@ void dev_mc146818_init(struct machine *machine, struct memory *mem,
 	 *  TODO:  Find out if this messes up Sprite emulation; if so, then
 	 *         this hack has to be removed.
 	 */
-	d->uip_threshold = 5;
+	d->uip_threshold = 8;
 
 	if (access_style == MC146818_ARC_JAZZ)
 		memory_device_register(mem, "mc146818_jazz", 0x90000070ULL,
