@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2003-2005  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2003-2006  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_pckbc.c,v 1.60 2005/11/25 04:25:26 debug Exp $
+ *  $Id: dev_pckbc.c,v 1.64 2006/01/01 13:17:16 debug Exp $
  *  
  *  Standard 8042 PC keyboard controller (and a 8242WB PS2 keyboard/mouse
  *  controller), including the 8048 keyboard chip.
@@ -582,12 +582,11 @@ static void dev_pckbc_command(struct pckbc_data *d, int port_nr)
 /*
  *  dev_pckbc_access():
  */
-int dev_pckbc_access(struct cpu *cpu, struct memory *mem,
-	uint64_t relative_addr, unsigned char *data, size_t len,
-	int writeflag, void *extra)
+DEVICE_ACCESS(pckbc)
 {
 	uint64_t idata = 0, odata = 0;
-	int i, port_nr = 0;
+	int port_nr = 0;
+	size_t i;
 	struct pckbc_data *d = extra;
 
 	if (writeflag == MEM_WRITE)
@@ -883,7 +882,7 @@ int dev_pckbc_init(struct machine *machine, struct memory *mem,
 	memset(d, 0, sizeof(struct pckbc_data));
 
 	if (type == PCKBC_8242)
-		len = 0x40;
+		len = 0x18;
 
 	if (type == PCKBC_JAZZ) {
 		type = PCKBC_8042;
@@ -895,13 +894,13 @@ int dev_pckbc_init(struct machine *machine, struct memory *mem,
 	d->mouse_irqnr       = mouse_irqnr;
 	d->in_use            = in_use;
 	d->pc_style_flag     = pc_style_flag;
-	if (d->in_use)
-		d->console_handle =
-		    console_start_slave_inputonly(machine, "pckbc");
 	d->translation_table = 2;
 	d->rx_int_enable     = 1;
 	d->output_byte       = 0x02;	/*  A20 enable on PCs  */
 
+	d->console_handle = console_start_slave_inputonly(
+	    machine, "pckbc", d->in_use);
+
 	memory_device_register(mem, "pckbc", baseaddr,
 	    len, dev_pckbc_access, d, DM_DEFAULT, NULL);
 	machine_add_tickfunction(machine, dev_pckbc_tick, d, PCKBC_TICKSHIFT);
