@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2003-2005  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2003-2006  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_ps2_gs.c,v 1.18 2005/11/13 00:14:09 debug Exp $
+ *  $Id: dev_ps2_gs.c,v 1.21 2006/02/09 20:02:59 debug Exp $
  *  
  *  Playstation 2 "graphics system".
  */
@@ -87,12 +87,11 @@ char *gs_reg_high_names[4] = {
 /*
  *  dev_ps2_gs_access():
  */
-int dev_ps2_gs_access(struct cpu *cpu, struct memory *mem,
-	uint64_t relative_addr, unsigned char *data, size_t len,
-	int writeflag, void *extra)
+DEVICE_ACCESS(ps2_gs)
 {
 	uint64_t idata = 0, odata = 0;
-	int i, regnr;
+	size_t i;
+	int regnr;
 	struct gs_data *d = extra;
 
 	if (writeflag == MEM_WRITE)
@@ -144,7 +143,7 @@ int dev_ps2_gs_access(struct cpu *cpu, struct memory *mem,
  *
  *  TODO:  Make this clearer.
  */
-int devinit_ps2_gs(struct devinit *devinit)
+DEVINIT(ps2_gs)
 {
 	struct gs_data *d;
 	char str[100];
