@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2005  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2004-2006  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_px.c,v 1.30 2005/11/13 00:14:09 debug Exp $
+ *  $Id: dev_px.c,v 1.33 2006/01/01 13:17:17 debug Exp $
  *  
  *  TURBOchannel Pixelstamp graphics device.
  *
@@ -143,7 +143,7 @@ uint32_t px_readword(struct cpu *cpu, unsigned char *dma_buf, int ofs)
 void dev_px_dma(struct cpu *cpu, uint32_t sys_addr, struct px_data *d)
 {
 	unsigned char dma_buf[32768];
-	int dma_len = sizeof(dma_buf);
+	size_t dma_len = sizeof(dma_buf);
 	int bytesperpixel;
 	uint32_t cmdword;
 
@@ -231,7 +231,7 @@ void dev_px_dma(struct cpu *cpu, uint32_t sys_addr, struct px_data *d)
 		debug(",HS_EQUALS");
 
 	{
-		int i;
+		size_t i;
 		for (i=0; i<dma_len; i++)
 			debug(" %02x", dma_buf[i]);
 	}
@@ -242,7 +242,7 @@ void dev_px_dma(struct cpu *cpu, uint32_t sys_addr, struct px_data *d)
 	/*  NetBSD and Ultrix copyspans  */
 	if (cmdword == 0x405) {
 		uint32_t nspans, lw;
-		int spannr, ofs;
+		uint32_t spannr, ofs;
 		uint32_t span_len, span_src, span_dst;
 		/*  unsigned char pixels[PX_XSIZE * 3];  */
 
@@ -328,20 +328,21 @@ void dev_px_dma(struct cpu *cpu, uint32_t sys_addr, struct px_data *d)
 
 			d->vfb_data->update_x1 = 0; d->vfb_data->update_x2 =
 			    PX_XSIZE-1;
-			if (span_dst < d->vfb_data->update_y1)
+			if ((int32_t)span_dst < d->vfb_data->update_y1)
 				d->vfb_data->update_y1 = span_dst;
-			if (span_dst > d->vfb_data->update_y2)
+			if ((int32_t)span_dst > d->vfb_data->update_y2)
 				d->vfb_data->update_y2 = span_dst;
-			if (span_src < d->vfb_data->update_y1)
+			if ((int32_t)span_src < d->vfb_data->update_y1)
 				d->vfb_data->update_y1 = span_src;
-			if (span_src > d->vfb_data->update_y2)
+			if ((int32_t)span_src > d->vfb_data->update_y2)
 				d->vfb_data->update_y2 = span_src;
 		}
 	}
 
 	/*  NetBSD and Ultrix erasecols/eraserows  */
 	if (cmdword == 0x411) {
-		uint32_t v1, v2, lw, attr;
+		uint32_t v1, v2, attr;
+		int32_t lw;
 		int x,y,x2,y2;
 		int fb_y;
 		int bg_r, bg_g, bg_b;
@@ -560,12 +561,11 @@ void dev_px_dma(struct cpu *cpu, uint32_t sys_addr, struct px_data *d)
 /*
  *  dev_px_access():
  */
-int dev_px_access(struct cpu *cpu, struct memory *mem, uint64_t relative_addr,
-	unsigned char *data, size_t len, int writeflag, void *extra)
+DEVICE_ACCESS(px)
 {
 	uint64_t idata = 0, odata = 0;
 	struct px_data *d = extra;
-	int i;
+	size_t i;
 
 	if (writeflag == MEM_WRITE)
 		idata = memory_readmax64(cpu, data, len);
