@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2005  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2004-2006  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_random.c,v 1.2 2005/11/13 00:14:09 debug Exp $
+ *  $Id: dev_random.c,v 1.5 2006/02/09 20:02:59 debug Exp $
  *  
  *  A simple device which returns random data for all reads, and discards all
  *  writes.
@@ -46,12 +46,10 @@
  *
  *  Returns 1 if ok, 0 on error.
  */
-int dev_random_access(struct cpu *cpu, struct memory *mem,
-	uint64_t relative_addr, unsigned char *data, size_t len,
-	int writeflag, void *extra)
+DEVICE_ACCESS(random)
 {
 	if (writeflag == MEM_READ) {
-		int i;
+		unsigned int i;
 		for (i=0; i<len; i++)
 			data[i] = random();
 	}
@@ -60,10 +58,7 @@ int dev_random_access(struct cpu *cpu, struct memory *mem,
 }
 
 
-/*
- *  devinit_random():
- */
-int devinit_random(struct devinit *devinit)
+DEVINIT(random)
 {
 	memory_device_register(devinit->machine->memory,
 	    devinit->name, devinit->addr, devinit->len,
