@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2005  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2004-2006  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_sgi_ip30.c,v 1.19 2005/11/13 00:14:10 debug Exp $
+ *  $Id: dev_sgi_ip30.c,v 1.20 2006/01/01 13:17:17 debug Exp $
  *  
  *  SGI IP30 stuff.
  *
@@ -60,9 +60,7 @@ void dev_sgi_ip30_tick(struct cpu *cpu, void *extra)
 /*
  *  dev_sgi_ip30_access():
  */
-int dev_sgi_ip30_access(struct cpu *cpu, struct memory *mem,
-	uint64_t relative_addr, unsigned char *data, size_t len,
-	int writeflag, void *extra)
+DEVICE_ACCESS(sgi_ip30)
 {
 	struct sgi_ip30_data *d = (struct sgi_ip30_data *) extra;
 	uint64_t idata = 0, odata = 0;
@@ -153,9 +151,7 @@ int dev_sgi_ip30_access(struct cpu *cpu, struct memory *mem,
 /*
  *  dev_sgi_ip30_2_access():
  */
-int dev_sgi_ip30_2_access(struct cpu *cpu, struct memory *mem,
-	uint64_t relative_addr, unsigned char *data, size_t len,
-	int writeflag, void *extra)
+DEVICE_ACCESS(sgi_ip30_2)
 {
 	struct sgi_ip30_data *d = (struct sgi_ip30_data *) extra;
 	uint64_t idata = 0, odata = 0;
@@ -208,9 +204,7 @@ int dev_sgi_ip30_2_access(struct cpu *cpu, struct memory *mem,
 /*
  *  dev_sgi_ip30_3_access():
  */
-int dev_sgi_ip30_3_access(struct cpu *cpu, struct memory *mem,
-	uint64_t relative_addr, unsigned char *data, size_t len,
-	int writeflag, void *extra)
+DEVICE_ACCESS(sgi_ip30_3)
 {
 	struct sgi_ip30_data *d = (struct sgi_ip30_data *) extra;
 	uint64_t idata = 0, odata = 0;
@@ -273,9 +267,7 @@ int dev_sgi_ip30_3_access(struct cpu *cpu, struct memory *mem,
 /*
  *  dev_sgi_ip30_4_access():
  */
-int dev_sgi_ip30_4_access(struct cpu *cpu, struct memory *mem,
-	uint64_t relative_addr, unsigned char *data, size_t len,
-	int writeflag, void *extra)
+DEVICE_ACCESS(sgi_ip30_4)
 {
 	struct sgi_ip30_data *d = (struct sgi_ip30_data *) extra;
 	uint64_t idata = 0, odata = 0;
@@ -319,9 +311,7 @@ int dev_sgi_ip30_4_access(struct cpu *cpu, struct memory *mem,
 /*
  *  dev_sgi_ip30_5_access():
  */
-int dev_sgi_ip30_5_access(struct cpu *cpu, struct memory *mem,
-	uint64_t relative_addr, unsigned char *data, size_t len,
-	int writeflag, void *extra)
+DEVICE_ACCESS(sgi_ip30_5)
 {
 	struct sgi_ip30_data *d = (struct sgi_ip30_data *) extra;
 	uint64_t idata = 0, odata = 0;
