@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2003-2005  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2003-2006  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_sgi_mardigras.c,v 1.20 2005/11/13 00:14:10 debug Exp $
+ *  $Id: dev_sgi_mardigras.c,v 1.22 2006/02/09 20:02:59 debug Exp $
  *  
  *  "MardiGras" graphics controller on SGI IP30 (Octane).
  *
@@ -212,9 +212,7 @@ void mardigras_20400(struct cpu *cpu, struct sgi_mardigras_data *d,
 /*
  *  dev_sgi_mardigras_access():
  */
-int dev_sgi_mardigras_access(struct cpu *cpu, struct memory *mem,
-	uint64_t relative_addr, unsigned char *data, size_t len,
-	int writeflag, void *extra)
+DEVICE_ACCESS(sgi_mardigras)
 {
 	uint64_t idata = 0, odata = 0;
 	struct sgi_mardigras_data *d = extra;
@@ -295,10 +293,7 @@ int dev_sgi_mardigras_access(struct cpu *cpu, struct memory *mem,
 }
 
 
-/*
- *  devinit_sgi_mardigras():
- */
-int devinit_sgi_mardigras(struct devinit *devinit)
+DEVINIT(sgi_mardigras)
 {
 	struct sgi_mardigras_data *d;
 
