@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2003-2005  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2003-2006  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_sii.c,v 1.15 2005/11/13 00:14:10 debug Exp $
+ *  $Id: dev_sii.c,v 1.16 2006/01/01 13:17:17 debug Exp $
  *  
  *  SII SCSI controller, used in some DECstation systems.
  *
@@ -179,9 +179,7 @@ void dev_sii_tick(struct cpu *cpu, void *extra)
 /*
  *  dev_sii_access():
  */
-int dev_sii_access(struct cpu *cpu, struct memory *mem,
-	uint64_t relative_addr, unsigned char *data, size_t len,
-	int writeflag, void *extra)
+DEVICE_ACCESS(sii)
 {
 	uint64_t idata = 0, odata = 0;
 	int regnr;
