@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2003-2005  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2003-2006  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_ssc.c,v 1.24 2005/11/13 00:14:10 debug Exp $
+ *  $Id: dev_ssc.c,v 1.26 2006/01/01 13:17:17 debug Exp $
  *  
  *  Serial controller on DECsystem 5400 and 5800.
  *  Known as System Support Chip on VAX 3600 (KA650).
@@ -114,9 +114,7 @@ void dev_ssc_tick(struct cpu *cpu, void *extra)
 /*
  *  dev_ssc_access():
  */
-int dev_ssc_access(struct cpu *cpu, struct memory *mem,
-	uint64_t relative_addr, unsigned char *data, size_t len,
-	int writeflag, void *extra)
+DEVICE_ACCESS(ssc)
 {
 	uint64_t idata = 0, odata = 0;
 	struct ssc_data *d = extra;
@@ -261,7 +259,7 @@ void dev_ssc_init(struct machine *machine, struct memory *mem,
 	d->irq_nr = irq_nr;
 	d->use_fb = use_fb;
 	d->csrp   = csrp;
-	d->console_handle = console_start_slave(machine, "SSC");
+	d->console_handle = console_start_slave(machine, "SSC", 1);
 
 	memory_device_register(mem, "ssc", baseaddr, DEV_SSC_LENGTH,
 	    dev_ssc_access, d, DM_DEFAULT, NULL);
