@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2005  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2004-2006  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_vr41xx.c,v 1.33 2005/11/13 00:14:10 debug Exp $
+ *  $Id: dev_vr41xx.c,v 1.35 2006/01/01 13:17:18 debug Exp $
  *  
  *  VR41xx (actually, VR4122 and VR4131) misc functions.
  *
@@ -389,9 +389,7 @@ static uint64_t vr41xx_kiu(struct cpu *cpu, int ofs, uint64_t idata,
 /*
  *  dev_vr41xx_access():
  */
-int dev_vr41xx_access(struct cpu *cpu, struct memory *mem,
-	uint64_t relative_addr, unsigned char *data, size_t len,
-	int writeflag, void *extra)
+DEVICE_ACCESS(vr41xx)
 {
 	struct vr41xx_data *d = (struct vr41xx_data *) extra;
 	uint64_t idata = 0, odata = 0;
@@ -559,7 +557,8 @@ struct vr41xx_data *dev_vr41xx_init(struct machine *machine,
 
 	/*  TODO: VRC4173 has the KIU at offset 0x100?  */
 	d->kiu_offset = 0x180;
-	d->kiu_console_handle = console_start_slave_inputonly(machine, "kiu");
+	d->kiu_console_handle = console_start_slave_inputonly(
+	    machine, "kiu", 1);
 	d->kiu_irq_nr = VRIP_INTR_KIU;
 
 	switch (cpumodel) {
