@@ -2,7 +2,7 @@
 #define	CPU_ALPHA_H
 
 /*
- *  Copyright (C) 2005  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2005-2006  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -28,7 +28,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: cpu_alpha.h,v 1.27 2005/11/16 21:15:19 debug Exp $
+ *  $Id: cpu_alpha.h,v 1.30 2006/02/09 22:40:27 debug Exp $
  */
 
 #include "misc.h"
@@ -124,6 +124,7 @@ struct alpha_cpu {
 	/*  Misc.:  */
 	uint64_t		pcc;			/*  Cycle Counter  */
 	uint64_t		ipl;
+	uint64_t		sysvalue;
 	uint64_t		load_linked_addr;
 	int			ll_flag;
 
@@ -131,22 +132,12 @@ struct alpha_cpu {
 	/*
 	 *  Instruction translation cache:
 	 */
-
-	/*  cur_ic_page is a pointer to an array of ALPHA_IC_ENTRIES_PER_PAGE
-	    instruction call entries. next_ic points to the next such
-	    call to be executed.  */
-	struct alpha_tc_physpage *cur_physpage;
-	struct alpha_instr_call	*cur_ic_page;
-	struct alpha_instr_call	*next_ic;
-
-	void			(*combination_check)(struct cpu *,
-				    struct alpha_instr_call *, int low_addr);
+	DYNTRANS_ITC(alpha)
 
 	/*
-	 *  Virtual -> physical -> host address translation:
+	 *  Hardcoded Alpha virtual -> physical -> host address translation:
 	 */
-
-	struct alpha_vpg_tlb_entry vph_tlb_entry[ALPHA_MAX_VPH_TLB_ENTRIES];
+	VPH_TLBS(alpha,ALPHA)
 	struct alpha_vph_page	*vph_default_page;
 	struct alpha_vph_page	*vph_next_free_page;
 	struct alpha_vph_table	*vph_next_free_table;
