@@ -2,7 +2,7 @@
 #define	CPU_HPPA_H
 
 /*
- *  Copyright (C) 2005  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2005-2006  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -28,7 +28,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: cpu_hppa.h,v 1.12 2005/11/16 21:15:19 debug Exp $
+ *  $Id: cpu_hppa.h,v 1.15 2006/02/13 04:23:25 debug Exp $
  */
 
 #include "misc.h"
@@ -40,73 +40,33 @@ struct cpu_family;
 #define	HPPA_N_IC_ARGS			3
 #define	HPPA_INSTR_ALIGNMENT_SHIFT	2
 #define	HPPA_IC_ENTRIES_SHIFT		10
-#define	HPPA_IC_ENTRIES_PER_PAGE		(1 << HPPA_IC_ENTRIES_SHIFT)
+#define	HPPA_IC_ENTRIES_PER_PAGE	(1 << HPPA_IC_ENTRIES_SHIFT)
 #define	HPPA_PC_TO_IC_ENTRY(a)		(((a)>>HPPA_INSTR_ALIGNMENT_SHIFT) \
 					& (HPPA_IC_ENTRIES_PER_PAGE-1))
 #define	HPPA_ADDR_TO_PAGENR(a)		((a) >> (HPPA_IC_ENTRIES_SHIFT \
 					+ HPPA_INSTR_ALIGNMENT_SHIFT))
 
-struct hppa_instr_call {
-	void	(*f)(struct cpu *, struct hppa_instr_call *);
-	size_t	arg[HPPA_N_IC_ARGS];
-};
+DYNTRANS_MISC_DECLARATIONS(hppa,HPPA,uint64_t)
 
-/*  Translation cache struct for each physical page:  */
-struct hppa_tc_physpage {
-	struct hppa_instr_call ics[HPPA_IC_ENTRIES_PER_PAGE + 1];
-	uint32_t	next_ofs;	/*  or 0 for end of chain  */
-	int		flags;
-	uint64_t	physaddr;
-};
+#define	HPPA_MAX_VPH_TLB_ENTRIES		128
 
-#define	HPPA_N_VPH_ENTRIES		1048576
 
-#define	HPPA_MAX_VPH_TLB_ENTRIES		256
-struct hppa_vpg_tlb_entry {
-	unsigned char	valid;
-	unsigned char	writeflag;
-	int64_t		timestamp;
-	uint64_t	vaddr_page;
-	uint64_t	paddr_page;
-	unsigned char	*host_page;
-};
+#define	HPPA_NREGS		32
 
 struct hppa_cpu {
 	int		bits;		/*  32 or 64  */
 
-	uint64_t	r[32];
+	uint64_t	r[HPPA_NREGS];
 
 
 	/*
-	 *  Instruction translation cache:
+	 *  Instruction translation cache and Virtual->Physical->Host
+	 *  address translation:
 	 */
-
-	/*  cur_ic_page is a pointer to an array of HPPA_IC_ENTRIES_PER_PAGE
-	    instruction call entries. next_ic points to the next such
-	    call to be executed.  */
-	struct hppa_tc_physpage	*cur_physpage;
-	struct hppa_instr_call	*cur_ic_page;
-	struct hppa_instr_call	*next_ic;
-
-	void			(*combination_check)(struct cpu *,
-				    struct hppa_instr_call *, int low_addr);
-
-	/*
-	 *  Virtual -> physical -> host address translation:
-	 *
-	 *  host_load and host_store point to arrays of HPPA_N_VPH_ENTRIES
-	 *  pointers (to host pages); phys_addr points to an array of
-	 *  HPPA_N_VPH_ENTRIES uint32_t.
-	 */
-
-	struct hppa_vpg_tlb_entry  vph_tlb_entry[HPPA_MAX_VPH_TLB_ENTRIES];
-	unsigned char		   *host_load[HPPA_N_VPH_ENTRIES]; 
-	unsigned char		   *host_store[HPPA_N_VPH_ENTRIES];
-	uint32_t		   phys_addr[HPPA_N_VPH_ENTRIES]; 
-	struct hppa_tc_physpage    *phys_page[HPPA_N_VPH_ENTRIES];
-
-	uint32_t		   phystranslation[HPPA_N_VPH_ENTRIES/32];
-	uint8_t			   vaddr_to_tlbindex[HPPA_N_VPH_ENTRIES];
+	DYNTRANS_ITC(hppa)
+	VPH_TLBS(hppa,HPPA)
+	VPH32(hppa,HPPA,uint64_t,uint8_t)
+	VPH64(hppa,HPPA,uint8_t)
 };
 
 
