@@ -2,7 +2,7 @@
 #define	CPU_I960_H
 
 /*
- *  Copyright (C) 2005  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2005-2006  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -28,7 +28,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: cpu_i960.h,v 1.7 2005/11/16 21:15:19 debug Exp $
+ *  $Id: cpu_i960.h,v 1.11 2006/02/13 04:23:25 debug Exp $
  */
 
 #include "misc.h"
@@ -49,31 +49,9 @@ struct cpu_family;
 #define	I960_ADDR_TO_PAGENR(a)		((a) >> (I960_IC_ENTRIES_SHIFT \
 					+ I960_INSTR_ALIGNMENT_SHIFT))
 
-struct i960_instr_call {
-	void	(*f)(struct cpu *, struct i960_instr_call *);
-	int	len;
-	size_t	arg[I960_N_IC_ARGS];
-};
+DYNTRANS_MISC_DECLARATIONS(i960,I960,uint32_t)
 
-/*  Translation cache struct for each physical page:  */
-struct i960_tc_physpage {
-	struct i960_instr_call ics[I960_IC_ENTRIES_PER_PAGE + 1];
-	uint32_t	next_ofs;	/*  or 0 for end of chain  */
-	uint32_t	physaddr;
-	int		flags;
-};
-
-
-#define	I960_N_VPH_ENTRIES	1048576
-
-#define	I960_MAX_VPH_TLB_ENTRIES		256
-struct i960_vpg_tlb_entry {
-	unsigned char	valid;
-	unsigned char	writeflag;
-	uint32_t	vaddr_page;
-	uint32_t	paddr_page;
-	unsigned char	*host_page;
-};
+#define	I960_MAX_VPH_TLB_ENTRIES		128
 
 
 struct i960_cpu {
@@ -85,35 +63,12 @@ struct i960_cpu {
 
 
 	/*
-	 *  Instruction translation cache:
+	 *  Instruction translation cache and 32-bit virtual -> physical ->
+	 *  host address translation:
 	 */
-
-	/*  cur_ic_page is a pointer to an array of I960_IC_ENTRIES_PER_PAGE
-	    instruction call entries. next_ic points to the next such
-	    call to be executed.  */
-	struct i960_tc_physpage	*cur_physpage;
-	struct i960_instr_call	*cur_ic_page;
-	struct i960_instr_call	*next_ic;
-
-	void			(*combination_check)(struct cpu *,
-				    struct i960_instr_call *, int low_addr);
-
-	/*
-	 *  Virtual -> physical -> host address translation:
-	 *
-	 *  host_load and host_store point to arrays of I960_N_VPH_ENTRIES
-	 *  pointers (to host pages); phys_addr points to an array of
-	 *  I960_N_VPH_ENTRIES uint32_t.
-	 */
-
-	struct i960_vpg_tlb_entry	vph_tlb_entry[I960_MAX_VPH_TLB_ENTRIES];
-	unsigned char			*host_load[I960_N_VPH_ENTRIES];
-	unsigned char			*host_store[I960_N_VPH_ENTRIES];
-	uint32_t			phys_addr[I960_N_VPH_ENTRIES];
-	struct i960_tc_physpage		*phys_page[I960_N_VPH_ENTRIES];
-
-	uint32_t			phystranslation[I960_N_VPH_ENTRIES/32];
-	uint8_t				vaddr_to_tlbindex[I960_N_VPH_ENTRIES];
+	DYNTRANS_ITC(i960)
+	VPH_TLBS(i960,I960)
+	VPH32(i960,I960,uint32_t,uint8_t)
 };
 
 
