@@ -2,7 +2,7 @@
 #define	CPU_X86_H
 
 /*
- *  Copyright (C) 2005  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2005-2006  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -28,7 +28,9 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: cpu_x86.h,v 1.39 2005/11/16 21:15:19 debug Exp $
+ *  $Id: cpu_x86.h,v 1.43 2006/02/13 04:23:25 debug Exp $
+ *
+ *  x86 (including AMD64) cpu dependent stuff.
  */
 
 #include "misc.h"
@@ -101,31 +103,9 @@ struct x86_model {
 #define	X86_PC_TO_IC_ENTRY(a)		((a) & (X86_IC_ENTRIES_PER_PAGE-1))
 #define	X86_ADDR_TO_PAGENR(a)		((a) >> X86_IC_ENTRIES_SHIFT)
 
-struct x86_instr_call {
-	void	(*f)(struct cpu *, struct x86_instr_call *);
-	int	len;
-	size_t	arg[X86_N_IC_ARGS];
-};
+DYNTRANS_MISC_DECLARATIONS(x86,X86,uint64_t)
 
-/*  Translation cache struct for each physical page:  */
-struct x86_tc_physpage {
-	struct x86_instr_call ics[X86_IC_ENTRIES_PER_PAGE + 1];
-	uint32_t	next_ofs;	/*  or 0 for end of chain  */
-	int		flags;
-	uint64_t	physaddr;
-};
-
-#define	X86_N_VPH_ENTRIES		1048576
-
-#define	X86_MAX_VPH_TLB_ENTRIES		256
-struct x86_vpg_tlb_entry {
-	uint8_t		valid;
-	uint8_t		writeflag;
-	unsigned char	*host_page;
-	int64_t		timestamp;
-	uint64_t	vaddr_page;
-	uint64_t	paddr_page;
-};
+#define	X86_MAX_VPH_TLB_ENTRIES		128
 
 struct descriptor_cache {
 	int		valid;
@@ -181,35 +161,13 @@ struct x86_cpu {
 
 
 	/*
-	 *  Instruction translation cache:
+	 *  Instruction translation cache and Virtual->Physical->Host
+	 *  address translation:
 	 */
-
-	/*  cur_ic_page is a pointer to an array of X86_IC_ENTRIES_PER_PAGE
-	    instruction call entries. next_ic points to the next such
-	    call to be executed.  */
-	struct x86_tc_physpage  *cur_physpage;
-	struct x86_instr_call   *cur_ic_page;
-	struct x86_instr_call   *next_ic;
-
-	void			(*combination_check)(struct cpu *,
-				    struct x86_instr_call *, int low_addr);
-
-	/*
-	 *  Virtual -> physical -> host address translation:
-	 *
-	 *  host_load and host_store point to arrays of X86_N_VPH_ENTRIES
-	 *  pointers (to host pages); phys_addr points to an array of
-	 *  X86_N_VPH_ENTRIES uint32_t.
-	 */
-
-	struct x86_vpg_tlb_entry        vph_tlb_entry[X86_MAX_VPH_TLB_ENTRIES];
-	unsigned char                   *host_load[X86_N_VPH_ENTRIES];
-	unsigned char                   *host_store[X86_N_VPH_ENTRIES];
-	uint32_t                        phys_addr[X86_N_VPH_ENTRIES];
-	struct x86_tc_physpage          *phys_page[X86_N_VPH_ENTRIES];
-
-	uint32_t			phystranslation[X86_N_VPH_ENTRIES/32];
-	uint8_t				vaddr_to_tlbindex[X86_N_VPH_ENTRIES];
+	DYNTRANS_ITC(x86)
+	VPH_TLBS(x86,X86)
+	VPH32(x86,X86,uint64_t,uint8_t)
+	VPH64(x86,X86,uint8_t)
 };
 
 
