@@ -28,7 +28,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: device.h,v 1.14 2005/11/22 16:26:38 debug Exp $
+ *  $Id: device.h,v 1.17 2006/02/09 20:02:59 debug Exp $
  *
  *  Device registry.  (See device.c for more info.)
  */
@@ -47,6 +47,7 @@ struct devinit {
 	uint64_t	addr2;		/*  Secondary address (optional)  */
 	uint64_t	len;
 	int		irq_nr;
+	int		dma_irq_nr;
 	int		in_use;
 	int		addr_mult;
 	int		pci_little_endian;
@@ -65,6 +66,8 @@ struct pci_entry {
 			    struct pci_device *);
 };
 
+#define	DEVINIT(name)		int devinit_ ## name (struct devinit *devinit)
+
 /*  autodev.c: (built automatically in the devices/ directory):  */
 void autodev_init(void);
 
@@ -80,7 +83,7 @@ void device_init(void);
 /*  PCI stuff:  (TODO: move somewhere else?)  */
 int pci_register(char *name, void (*initf)(struct machine *, struct memory *,
 	struct pci_device *));
-void (*pci_lookup_initf(char *name))(struct machine *machine,
+void (*pci_lookup_initf(const char *name))(struct machine *machine,
 	struct memory *mem, struct pci_device *pd);
 
 #endif	/*  DEVICE_H  */
