@@ -28,7 +28,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: diskimage.h,v 1.29 2005/09/27 23:55:46 debug Exp $
+ *  $Id: diskimage.h,v 1.30 2005/11/13 02:22:01 debug Exp $
  *
  *  Generic disk image functions.  (See diskimage.c for more info.)
  */
@@ -139,6 +139,8 @@ void diskimage_dump_info(struct machine *machine);
 #define	SCSICMD_MODE_SELECT		0x15
 #define	SCSICMD_MODE_SENSE		0x1a
 #define	SCSICMD_START_STOP_UNIT		0x1b
+#define	SCSICMD_PREVENT_ALLOW_REMOVE	0x1e
+#define	SCSICMD_MODE_SENSE10		0x5a
 
 #define	SCSICMD_SYNCHRONIZE_CACHE	0x35
 
