@@ -2,7 +2,7 @@
 #define	MACHINE_H
 
 /*
- *  Copyright (C) 2005  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2005-2006  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -28,12 +28,13 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: machine.h,v 1.86 2005/11/22 02:07:40 debug Exp $
+ *  $Id: machine.h,v 1.105 2006/02/18 13:42:39 debug Exp $
  */
 
 #include <sys/types.h>
 #include <sys/time.h>
 
+
 #include "symbol.h"
 
 #include "machine_arc.h"
@@ -50,6 +51,7 @@ struct diskimage;
 struct emul;
 struct fb_window;
 struct memory;
+struct of_data;
 
 /*  Ugly:  */
 struct kn230_csr;
@@ -75,6 +77,16 @@ struct isa_pic_data {
 };
 
 
+struct machine_bus {
+	struct machine_bus *next;
+
+	char		*name;
+
+	void		(*debug_dump)(void *);
+	void		*extra;
+};
+
+
 #define	MACHINE_NAME_MAXBUF		150
 
 struct machine {
@@ -85,11 +97,15 @@ struct machine {
 	char	*name;
 
 	int	arch;			/*  ARCH_MIPS, ARCH_PPC, ..  */
-	int	machine_type;		/*  MACHINE_DEC, ..  */
+	int	machine_type;		/*  MACHINE_PMAX, ..  */
 	int	machine_subtype;	/*  MACHINE_DEC_3MAX_5000, ..  */
 
+	/*  Name set by code in src/machines/machine_*.c:  */
 	char	*machine_name;
 
+	int	stable;			/*  startup warning for non-stable
+					    emulation modes.  */
+
 	/*  The serial number is mostly used when emulating multiple machines
 	    in a network. nr_of_nics is the current nr of network cards, which
 	    is useful when emulating multiple cards in one machine:  */
@@ -129,6 +145,10 @@ struct machine {
 	int	ncpus;
 	struct cpu **cpus;
 
+	/*  Registered busses:  */
+	struct machine_bus *first_bus;
+	int	n_busses;
+
 	/*  These are used by stuff in cpu.c, mostly:  */
 	int64_t ncycles;
 	int64_t	ncycles_show;
@@ -149,6 +169,11 @@ struct machine {
 	int	register_dump;
 	int	arch_pagesize;
 
+	int	bootdev_type;
+	int	bootdev_id;
+	char	*bootstr;
+	char	*bootarg;
+
 	int	n_breakpoints;
 	char	*breakpoint_string[MAX_BREAKPOINTS];
 	uint64_t breakpoint_addr[MAX_BREAKPOINTS];
@@ -168,7 +193,7 @@ struct machine {
 	int	bintrans_enable;
 	int	old_bintrans_enable;
 	int	bintrans_enabled_from_start;
-	int	bintrans_size;
+	size_t	bintrans_size;
 	int	instruction_trace;
 	int	single_step_on_bad_addr;
 	int	show_nr_of_instructions;
@@ -196,6 +221,9 @@ struct machine {
 		struct machine_pc	pc;
 	} md;
 
+	/*  OpenFirmware:  */
+	struct of_data *of_data;
+
 	/*  Bus-specific interrupt data:  */
 	struct isa_pic_data isa_pic_data;
 
@@ -221,6 +249,8 @@ struct machine {
 		struct bebox_data *bebox_data;
 		struct prep_data *prep_data;
 		struct cpc700_data *cpc700_data;
+		struct gc_data *gc_data;
+		struct v3_data *v3_data;
 	} md_int;
 
 	/*  X11/framebuffer stuff:  */
@@ -253,12 +283,11 @@ struct machine {
 #define	ARCH_HPPA		10
 #define	ARCH_I960		11
 #define	ARCH_AVR		12
-#define	ARCH_NEWMIPS		13
 
 /*  MIPS:  */
 #define	MACHINE_BAREMIPS	1000
 #define	MACHINE_TESTMIPS	1001
-#define	MACHINE_DEC		1002
+#define	MACHINE_PMAX		1002
 #define	MACHINE_COBALT		1003
 #define	MACHINE_HPCMIPS		1004
 #define	MACHINE_PS2		1005
@@ -281,6 +310,7 @@ struct machine {
 #define	MACHINE_PREP		2006
 #define	MACHINE_MACPPC		2007
 #define	MACHINE_DB64360		2008
+#define	MACHINE_MVMEPPC		2009
 
 /*  SPARC:  */
 #define	MACHINE_BARESPARC	3000
@@ -306,6 +336,7 @@ struct machine {
 #define	MACHINE_SHARK		6006
 #define	MACHINE_IQ80321		6007
 #define	MACHINE_IYONIX		6008
+#define	MACHINE_TS7200		6009
 
 /*  IA64:  */
 #define	MACHINE_BAREIA64	7000
@@ -331,10 +362,6 @@ struct machine {
 /*  AVR:  */
 #define	MACHINE_BAREAVR		12000
 
-/*  NEWMIPS:  */
-#define	MACHINE_BARENEWMIPS	13000
-#define	MACHINE_TESTNEWMIPS	13001
-
 /*  Other "pseudo"-machines:  */
 #define	MACHINE_NONE		0
 #define	MACHINE_USERLAND	100000
@@ -399,10 +426,19 @@ struct machine {
 #define	MACHINE_EVBMIPS_MALTA_BE	2
 #define	MACHINE_EVBMIPS_PB1000		3
 
+/*  PReP:  */
+#define	MACHINE_PREP_IBM6050		1
+#define	MACHINE_PREP_MVME2400		2
+
 /*  MacPPC:  TODO: Real model names  */
 #define	MACHINE_MACPPC_G4		1
 #define	MACHINE_MACPPC_G5		2
 
+/*  MVMEPPC  */
+#define	MACHINE_MVMEPPC_1600		1
+#define	MACHINE_MVMEPPC_2100		2
+#define	MACHINE_MVMEPPC_5500		3
+
 /*  X86:  */
 #define	MACHINE_X86_GENERIC		1
 #define	MACHINE_X86_XT			2
@@ -430,12 +466,53 @@ struct machine {
 #define	ARC_PRIVATE_ENTRIES	0xffffffffbfcb8000ULL
 
 
+/*  For the automachine system:  */
+struct machine_entry_subtype {
+	int			machine_subtype;/*  Old-style subtype  */
+	const char		*name;		/*  Official name  */
+	int			n_aliases;
+	char			**aliases;	/*  Aliases  */
+};
+
+struct machine_entry {
+	struct machine_entry	*next;
+
+	/*  Machine type:  */
+	int			arch;
+	int			machine_type;	/*  Old-style type  */
+	const char		*name;		/*  Official name  */
+	int			n_aliases;
+	char			**aliases;	/*  Aliases  */
+
+	void			(*setup)(struct machine *, struct cpu *);
+	void			(*set_default_cpu)(struct machine *);
+	void			(*set_default_ram)(struct machine *);
+
+	/*  Machine subtypes:  */
+	int			n_subtypes;
+	struct machine_entry_subtype **subtype;
+};
+
+#define	MACHINE_SETUP_TYPE(n)	void (*n)(struct machine *, struct cpu *)
+#define	MACHINE_SETUP(x)	void machine_setup_ ## x(struct machine *machine, \
+				    struct cpu *cpu)
+#define	MACHINE_DEFAULT_CPU(x)	void machine_default_cpu_ ## x(struct machine *machine)
+#define	MACHINE_DEFAULT_RAM(x)	void machine_default_ram_ ## x(struct machine *machine)
+#define	MACHINE_REGISTER(x)	void machine_register_ ## x(void)
+#define	MR_DEFAULT(x,name,arch,type,n,m) struct machine_entry \
+	    *me = machine_entry_new(name,arch,type,n,m);	\
+	me->setup = machine_setup_ ## x;	\
+	me->set_default_cpu = machine_default_cpu_ ## x;
+void automachine_init(void);
+
+
 /*  machine.c:  */
 struct machine *machine_new(char *name, struct emul *emul);
 int machine_name_to_type(char *stype, char *ssubtype,
 	int *type, int *subtype, int *arch);
 void machine_add_tickfunction(struct machine *machine,
 	void (*func)(struct cpu *, void *), void *extra, int clockshift);
+void machine_register(char *name, MACHINE_SETUP_TYPE(setup));
 void dump_mem_string(struct cpu *cpu, uint64_t addr);
 void store_string(struct cpu *cpu, uint64_t addr, char *s);
 int store_64bit_word(struct cpu *cpu, uint64_t addr, uint64_t data64);
@@ -451,11 +528,23 @@ void store_16bit_word_in_host(struct cpu *cpu, unsigned char *data,
 uint32_t load_32bit_word(struct cpu *cpu, uint64_t addr);
 uint16_t load_16bit_word(struct cpu *cpu, uint64_t addr);
 void store_buf(struct cpu *cpu, uint64_t addr, char *s, size_t len);
+void add_environment_string(struct cpu *cpu, char *s, uint64_t *addr);
+void add_environment_string_dual(struct cpu *cpu,
+	uint64_t *ptrp, uint64_t *addrp, char *s1, char *s2);
+void store_pointer_and_advance(struct cpu *cpu, uint64_t *addrp,
+	uint64_t data, int flag64);
 void machine_setup(struct machine *);
 void machine_memsize_fix(struct machine *);
 void machine_default_cputype(struct machine *);
 void machine_dumpinfo(struct machine *);
+void machine_bus_register(struct machine *, char *busname,
+	void (*debug_dump)(void *), void *extra);
 void machine_list_available_types_and_cpus(void);
+struct machine_entry *machine_entry_new(const char *name, 
+	int arch, int oldstyle_type, int n_aliases, int n_subtypes);
+struct machine_entry_subtype *machine_entry_subtype_new(
+	const char *name, int oldstyle_type, int n_aliases);
+void machine_entry_add(struct machine_entry *me, int arch);
 void machine_init(void);
 
 
