@@ -28,7 +28,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: opcodes_ppc.h,v 1.42 2005/11/24 01:15:07 debug Exp $
+ *  $Id: opcodes_ppc.h,v 1.45 2005/12/04 02:40:04 debug Exp $
  *
  *
  *  PPC and POWER opcodes. These are gathered from various sources.
@@ -99,6 +99,7 @@
 #define	  PPC_31_LDARX		  84
 #define	  PPC_31_DCBF		  86
 #define	  PPC_31_LBZX		  87
+#define	  PPC_31_LVX		  103
 #define	  PPC_31_NEG		  104
 #define	  PPC_31_CLF		  118
 #define	  PPC_31_LBZUX		  119
@@ -110,6 +111,7 @@
 #define	  PPC_31_STDX		  149
 #define	  PPC_31_STWCX_DOT	  150
 #define	  PPC_31_STWX		  151
+#define	  PPC_31_WRTEEI		  163
 #define	  PPC_31_STDUX		  181
 #define	  PPC_31_STWUX		  183
 #define	  PPC_31_SUBFZE		  200
@@ -117,6 +119,7 @@
 #define	  PPC_31_MTSR		  210
 #define	  PPC_31_STDCX_DOT	  214
 #define	  PPC_31_STBX		  215
+#define	  PPC_31_STVX		  231
 #define	  PPC_31_SUBFME		  232
 #define	  PPC_31_ADDME		  234
 #define	  PPC_31_MULLW		  235
@@ -144,6 +147,7 @@
 #define	  PPC_31_MTSPR		  467
 #define	  PPC_31_DCBI		  470
 #define	  PPC_31_NAND		  476
+#define	  PPC_31_STVXL		  487
 #define	  PPC_31_DIVW		  491
 #define	  PPC_31_SLBIA		  498
 #define	  PPC_31_CLI		  502
@@ -177,6 +181,7 @@
 #define	  PPC_31_SRAW		  792
 #define	  PPC_31_SRAWI		  824
 #define	  PPC_31_EIEIO		  854
+#define	  PPC_31_TLBSX_DOT	  914
 #define	  PPC_31_STHBRX		  918
 #define	  PPC_31_EXTSH		  922
 #define	  PPC_31_EXTSB		  954
