@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2003-2005  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2003-2006  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: main.c,v 1.248 2005/11/23 02:17:00 debug Exp $
+ *  $Id: main.c,v 1.256 2006/02/04 11:10:58 debug Exp $
  */
 
 #include <stdio.h>
@@ -56,6 +56,7 @@ char **extra_argv;
 char *progname;
 
 int fully_deterministic = 0;
+int dyntrans_backend_enable = 1;
 
 
 /*****************************************************************************
@@ -194,7 +195,7 @@ static void usage(int longusage)
 #ifdef VERSION
 	printf("-" VERSION);
 #endif
-	printf("   Copyright (C) 2003-2005  Anders Gavare\n");
+	printf("   Copyright (C) 2003-2006  Anders Gavare\n");
 	printf("Read the source code and/or documentation for "
 	    "other Copyright messages.\n");
 
@@ -219,10 +220,10 @@ static void usage(int longusage)
 	printf("\nOther options:\n");
 	printf("  -A        disable alignment checks in some cases (for higher"
 	    " speed)\n");
-#ifdef BINTRANS
-	printf("  -B        disable dynamic binary translation. (translation"
-	    " is turned on\n            by default, if the host "
-	    "supports it)\n");
+#if defined(BINTRANS) || defined(DYNTRANS_BACKEND)
+	printf("  -B        disable native translation backends. (translation"
+	    " is turned on\n            by default, if it is supposed for "
+	    "the particular host)\n");
 #endif
 	printf("  -C x      try to emulate a specific CPU. (Use -H to get a "
 	    "list of types.)\n");
@@ -279,8 +280,8 @@ static void usage(int longusage)
 	printf("  -X        use X11\n");
 #endif /*  WITH_X11  */
 	printf("  -x        open up new xterms for emulated serial ports "
-	    "(default is on when\n            using configuration files, off"
-	    " otherwise)\n");
+	    "(default is on when\n            using configuration files or"
+	    " when X11 is used, off otherwise)\n");
 #ifdef WITH_X11
 	printf("  -Y n      scale down framebuffer windows by n x n times\n");
 #endif /*  WITH_X11  */
@@ -297,6 +298,8 @@ static void usage(int longusage)
 	    " get a list of\n            available emulation modes)\n");
 
 	printf("\nGeneral options:\n");
+	printf("  -c cmd    add cmd as a command to run before starting "
+	    "the simulation\n");
 	printf("  -D        guarantee fully deterministic behaviour\n");
 	printf("  -H        display a list of possible CPU and "
 	    "machine types\n");
@@ -340,7 +343,7 @@ int get_cmd_args(int argc, char *argv[], struct emul *emul,
 	int msopts = 0;		/*  Machine-specific options used  */
 	struct machine *m = emul_add_machine(emul, "default");
 
-	while ((ch = getopt(argc, argv, "ABC:Dd:E:e:HhI:iJj:KM:m:"
+	while ((ch = getopt(argc, argv, "ABC:c:Dd:E:e:HhI:iJj:KM:m:"
 	    "Nn:Oo:p:QqRrSsTtUu:VvW:XxY:y:Z:z:")) != -1) {
 		switch (ch) {
 		case 'A':
@@ -348,13 +351,26 @@ int get_cmd_args(int argc, char *argv[], struct emul *emul,
 			msopts = 1;
 			break;
 		case 'B':
+			/*  Turns off both bintrans and dyntrans.  */
 			m->bintrans_enable = 0;
+			dyntrans_backend_enable = 0;
 			msopts = 1;
 			break;
 		case 'C':
 			m->cpu_name = strdup(optarg);
 			msopts = 1;
 			break;
+		case 'c':
+			emul->n_debugger_cmds ++;
+			emul->debugger_cmds = realloc(emul->debugger_cmds,
+			    emul->n_debugger_cmds * sizeof(char *));
+			if (emul->debugger_cmds == NULL) {
+			        fatal("out of memory\n");
+			        exit(1);
+			}
+			emul->debugger_cmds[emul->n_debugger_cmds-1] =
+			    strdup(optarg);
+			break;
 		case 'D':
 			fully_deterministic = 1;
 			break;
@@ -507,7 +523,7 @@ int get_cmd_args(int argc, char *argv[], struct emul *emul,
 		case 'X':
 			m->use_x11 = 1;
 			msopts = 1;
-			break;
+			/*  FALL-THROUGH  */
 		case 'x':
 			console_allow_slaves(1);
 			break;
@@ -651,10 +667,12 @@ int get_cmd_args(int argc, char *argv[], struct emul *emul,
 		exit(1);
 	}
 
+#ifndef EXPERIMENTAL_NEWMIPS
 	if (m->bintrans_enable && m->arch == ARCH_MIPS) {
 		m->speed_tricks = 0;
 		/*  TODO: Print a warning about this?  */
 	}
+#endif
 
 	if (m->n_breakpoints > 0 &&
 	    m->bintrans_enable && m->arch == ARCH_MIPS) {
@@ -746,7 +764,7 @@ int main(int argc, char *argv[])
 #ifdef VERSION
 	debug("-" VERSION);
 #endif
-	debug("   Copyright (C) 2003-2005  Anders Gavare\n");
+	debug("   Copyright (C) 2003-2006  Anders Gavare\n");
 	debug("Read the source code and/or documentation for "
 	    "other Copyright messages.\n\n");
 
@@ -789,12 +807,13 @@ int main(int argc, char *argv[])
 				fprintf(stderr, "out of memory\n");
 				exit(1);
 			}
-			emuls[n_emuls - 1] =
-			    emul_create_from_configfile(s);
 
 			/*  Always allow slave xterms when using multiple
 			    emulations:  */
 			console_allow_slaves(1);
+
+			emuls[n_emuls - 1] =
+			    emul_create_from_configfile(s);
 		}
 	}
 
@@ -808,7 +827,7 @@ int main(int argc, char *argv[])
 	}
 
 	device_set_exit_on_error(0);
-	console_warn_if_slaves_are_needed();
+	console_warn_if_slaves_are_needed(1);
 
 	/*  Run all emulations:  */
 	emul_run(emuls, n_emuls);
