@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2003-2005  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2003-2006  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: dec_prom.c,v 1.1 2005/08/29 14:46:34 debug Exp $
+ *  $Id: dec_prom.c,v 1.4 2006/02/16 05:57:10 debug Exp $
  *
  *  DECstation PROM emulation.
  */
@@ -349,7 +349,7 @@ int decstation_prom_emul(struct cpu *cpu)
 		i = 0; ch = -1; argreg = MIPS_GPR_A1;
 		while (ch != '\0') {
 			char printfbuf[8000];
-			int x;
+			size_t x;
 
 			printfbuf[0] = printfbuf[sizeof(printfbuf)-1] = '\0';
 
@@ -508,7 +508,7 @@ int decstation_prom_emul(struct cpu *cpu)
 		break;
 	case 0x64:		/*  getenv()  */
 		/*  Find the environment variable given by a0:  */
-		for (i=0; i<sizeof(buf); i++)
+		for (i=0; i<(int)sizeof(buf); i++)
 			cpu->memory_rw(cpu, cpu->mem,
 			    cpu->cd.mips.gpr[MIPS_GPR_A0] + i, &buf[i],
 			    sizeof(char), MEM_READ, CACHE_DATA | NO_EXCEPTIONS);
@@ -517,7 +517,7 @@ int decstation_prom_emul(struct cpu *cpu)
 		for (i=0; i<0x1000; i++) {
 			/*  Matching string at offset i?  */
 			int nmatches = 0;
-			for (j=0; j<strlen((char *)buf); j++) {
+			for (j=0; j<(int32_t)strlen((char *)buf); j++) {
 				cpu->memory_rw(cpu, cpu->mem, (uint64_t)
 				    (DEC_PROM_STRINGS + i + j), &ch2,
 				    sizeof(char), MEM_READ, CACHE_DATA |
@@ -529,7 +529,7 @@ int decstation_prom_emul(struct cpu *cpu)
 			    (uint64_t)(DEC_PROM_STRINGS
 			    + i + strlen((char *)buf)), &ch2, sizeof(char),
 			    MEM_READ, CACHE_DATA | NO_EXCEPTIONS);
-			if (nmatches == strlen((char *)buf) && ch2 == '=') {
+			if (nmatches == (int)strlen((char *)buf) && ch2=='=') {
 				cpu->cd.mips.gpr[MIPS_GPR_V0] =
 				    DEC_PROM_STRINGS + i +
 				    strlen((char *)buf) + 1;
