@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2005  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2005-2006  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: pc_bios.c,v 1.3 2005/11/05 13:39:42 debug Exp $
+ *  $Id: pc_bios.c,v 1.5 2006/02/16 05:57:10 debug Exp $
  *
  *  Generic PC BIOS emulation.
  *
@@ -1330,7 +1330,7 @@ static void pc_bios_int1a(struct cpu *cpu)
 static void pc_bios_int1c(struct cpu *cpu)
 {
 	unsigned char ticks[4];
-	int i;
+	size_t i;
 
 	/*  Increase word at 0x0040:0x006C  */
 	cpu->memory_rw(cpu, cpu->mem, 0x46C,
