@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2005  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2004-2006  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: useremul.c,v 1.66 2005/11/11 13:23:15 debug Exp $
+ *  $Id: useremul.c,v 1.70 2006/01/14 12:51:59 debug Exp $
  *
  *  Userland (syscall) emulation.
  *
@@ -364,7 +364,7 @@ static unsigned char *get_userland_buf(struct cpu *cpu,
 	uint64_t baseaddr, uint64_t len)
 {
 	unsigned char *charbuf;
-	ssize_t i;
+	size_t i;
 
 	charbuf = malloc(len);
 	if (charbuf == NULL) {
@@ -699,6 +699,10 @@ static void useremul__freebsd(struct cpu *cpu, uint32_t code)
 	case 58:res = useremul_readlink(cpu, &err, arg0, arg1, arg2);
 		break;
 
+	case 73:/* munmap. TODO */
+		res = 1;
+		break;
+
 	case 117:res = useremul_getrusage(cpu, &err, arg0, arg1);
 		break;
 
@@ -1712,7 +1716,7 @@ static void add_useremul(char *name, int arch, char *cpu_name,
 void useremul_list_emuls(void)
 {
 	struct syscall_emul *sep;
-	int iadd = 8;
+	int iadd = DEBUG_INDENTATION * 2;
 
 	sep = first_syscall_emul;
 
