@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2003-2005  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2003-2006  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: x11.c,v 1.59 2005/06/20 05:52:47 debug Exp $
+ *  $Id: x11.c,v 1.61 2006/01/14 12:51:59 debug Exp $
  *
  *  X11-related functions.
  */
@@ -67,6 +67,8 @@ void x11_check_event(struct emul **emuls, int n_emuls) { }
  *  x11_redraw_cursor():
  *
  *  Redraw a framebuffer's X11 cursor.
+ *
+ *  NOTE: It is up to the caller to call XFlush.
  */
 void x11_redraw_cursor(struct machine *m, int i)
 {
@@ -178,7 +180,6 @@ void x11_redraw_cursor(struct machine *m, int i)
 		    m->fb_windows[i]->cursor_xsize;
 		m->fb_windows[i]->OLD_cursor_ysize =
 		    m->fb_windows[i]->cursor_ysize;
-		XFlush(m->fb_windows[i]->x11_display);
 	}
 
 	/*  printf("n_colors_used = %i\n", n_colors_used);  */
@@ -187,7 +188,6 @@ void x11_redraw_cursor(struct machine *m, int i)
 		/*  Remove the old X11 host cursor:  */
 		XUndefineCursor(m->fb_windows[i]->x11_display,
 		    m->fb_windows[i]->x11_fb_window);
-		XFlush(m->fb_windows[i]->x11_display);
 		XFreeCursor(m->fb_windows[i]->x11_display,
 		    m->fb_windows[i]->host_cursor);
 		m->fb_windows[i]->host_cursor = 0;
@@ -231,7 +231,6 @@ void x11_redraw_cursor(struct machine *m, int i)
 			XDefineCursor(m->fb_windows[i]->x11_display,
 			    m->fb_windows[i]->x11_fb_window,
 			    m->fb_windows[i]->host_cursor);
-			XFlush(m->fb_windows[i]->x11_display);
 		}
 	}
 }
